// i2c - AQM0802A 

#include <stdio.h>
#include <string.h>

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "AQM0802A.h"

// i2c master initialization
esp_err_t i2c_master_init(void)
{
    int i2c_master_port = I2C_MASTER_NUM;

    i2c_config_t conf = {
        .mode = I2C_MODE_MASTER,
        .sda_io_num = I2C_MASTER_SDA_IO,
        .scl_io_num = I2C_MASTER_SCL_IO,
        .sda_pullup_en = GPIO_PULLUP_ENABLE,
        .scl_pullup_en = GPIO_PULLUP_ENABLE,
        .master.clk_speed = I2C_MASTER_FREQ_HZ,
    };

    i2c_param_config(i2c_master_port, &conf);

    return i2c_driver_install(i2c_master_port, conf.mode, I2C_MASTER_RX_BUF_DISABLE, I2C_MASTER_TX_BUF_DISABLE, 0);
}

// Write a byte to a AQM0802A 
esp_err_t AQM0802A_write_byte(uint8_t data, uint8_t reg_addr)
{
    int ret;
    uint8_t write_buf[2] = {reg_addr, data};

    ret = i2c_master_write_to_device(I2C_MASTER_NUM, AQM0802A_ADDR, write_buf, sizeof(write_buf), I2C_MASTER_TIMEOUT_MS / portTICK_RATE_MS);

    return ret;
}

// Command write
void AQM0802A_command(uint8_t value)
{
  	AQM0802A_write_byte(value, (uint8_t)0x00);
}

// Data write
void AQM0802A_data(uint8_t value)
{
	AQM0802A_write_byte(value, (uint8_t)0x40);
}

void AQM0802A_init(void)
{
	uint8_t value = 0x00;
  	
  	// Initializing
  	// ￼Wait time >40mS After VDD stable
	vTaskDelay(40 / portTICK_PERIOD_MS);

  	//Function Set: Normal mode:
  	//  Dl: interface data 8/4bits
  	//  N: number of line 2/1
  	//  001D Nxxx
  	//  0011 1000 : 0x38
  	value = 0x38; 
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	//Function Set: Extension mode:
  	//  Dl: interface data 8/4bits
  	//  N: number of line 2/1
  	//  dH: double height font
  	//  Is: instruction table select
  	//  001D NH0I
  	//  0011 1001 : 0x39
  	value = 0x39; 
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	//Internal OSC frequency:
  	//  Bs: 1=1/4 bias 0=1/5 bias
  	//  F2-0: adjust internal OSC frequency for FR frequency.
  	//  0001 BFFF
  	//  0001 0100 : 0x14
  	value = 0x14;
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	//Contrast set:
  	//  C3-0: Contrast set for internal follower mode.
  	//  0111 CCCC
  	//  0111 0000 : 0x70
  	value = 0x70;
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	//Power/ICON control/Contrast set:
  	//  Ion: ICON display on/off
  	//  Bon: set booster circuit on/off
  	//  C5-4: Contrast set for internal follower mode.
  	//  0101 IBCC
  	//  0101 0110 : 0x56
  	value = 0x56;
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	// Contrast initialize: xx10 0000(32d)

  	//Follower control:
  	//  Fon: set follower circuit on/off
  	//  Rab2-0: select follower amplified ratio.
  	//  0110 FRRR
  	//  0101 1100 : 0x6C
  	value = 0x6C;
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	//Function Set: Normal mode:
  	//  Dl: interface data 8/4bits
  	//  N: number of line 2/1
  	//  001D Nxxx
  	//  0011 1000 : 0x38
  	value = 0x38;
  	AQM0802A_command(value);
  	// Wait time >26.3uS

  	// turn the display on with no cursor or blinking default
  	_displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;

  	// Initialize to default text direction (for romance languages)
  	_displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;

  	AQM0802A_display();
  	AQM0802A_clear();
  	AQM0802A_home();
}

void AQM0802A_print(char* buf)
{
	int a;
	for(a = 0; a < strlen(buf); a++) AQM0802A_data(buf[a]);
}

void AQM0802A_clear()
{
  	AQM0802A_command(LCD_CLEARDISPLAY);  		// clear display, set cursor position to zero
  	vTaskDelay(200 / portTICK_PERIOD_MS);  		// this command takes a long time!
}

void AQM0802A_home()
{
  	AQM0802A_command(LCD_RETURNHOME);  			// set cursor position to zero
  	vTaskDelay(200 / portTICK_PERIOD_MS);  		// this command takes a long time!
}

void AQM0802A_setCursor(uint8_t col, uint8_t row)
{
	if(row) col += 0x40;
  	AQM0802A_command(LCD_SETDDRAMADDR | col);
}

void AQM0802A_noDisplay()
{
  	_displaycontrol &= ~LCD_DISPLAYON;
  	AQM0802A_command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void AQM0802A_display()
{
  	_displaycontrol |= LCD_DISPLAYON;
  	AQM0802A_command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void AQM0802A_noCursor()
{
  	_displaycontrol &= ~LCD_CURSORON;
  	AQM0802A_command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void AQM0802A_cursor()
{
  	_displaycontrol |= LCD_CURSORON;
  	AQM0802A_command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void AQM0802A_noBlink()
{
  	_displaycontrol &= ~LCD_BLINKON;
  	AQM0802A_command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void AQM0802A_blink()
{
  	_displaycontrol |= LCD_BLINKON;
  	AQM0802A_command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void AQM0802A_scrollDisplayLeft(void)
{
  	AQM0802A_command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}

void AQM0802A_scrollDisplayRight(void)
{
  	AQM0802A_command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}

void AQM0802A_leftToRight(void)
{
  	_displaymode |= LCD_ENTRYLEFT;
  	AQM0802A_command(LCD_ENTRYMODESET | _displaymode);
}

void AQM0802A_rightToLeft(void)
{
  	_displaymode &= ~LCD_ENTRYLEFT;
  	AQM0802A_command(LCD_ENTRYMODESET | _displaymode);
}

void AQM0802A_autoscroll(void)
{
  	_displaymode |= LCD_ENTRYSHIFTINCREMENT;
  	AQM0802A_command(LCD_ENTRYMODESET | _displaymode);
}

void AQM0802A_noAutoscroll(void)
{
  	_displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
  	AQM0802A_command(LCD_ENTRYMODESET | _displaymode);
}

void AQM0802A_createChar(uint8_t location, uint8_t charmap[])
{
  	location &= 0x7; // we only have 8 locations 0-7
  	AQM0802A_command(LCD_SETCGRAMADDR | (location << 3));
  	for (int i=0; i<8; i++) AQM0802A_data(charmap[i]);
}


